indexing

	description: "ISE routines to access its garbage collector."

	date: "$Date: 2003/11/29 $"
	revision: "$Revision: #2 $"


class

	EPX_ISE_GC_HELPER


feature -- Unfreezing

	unfreeze_objects is
			-- Unfreeze the objects in `freezed_objects'.
		local
			i: INTEGER
		do
			-- This is guaranteed in eposix:
				check
					garbage_collector_enabled: highest_freezed_object.item > 0 implies memory.collecting
				end
			from
				i := freezed_objects.lower
			variant
				freezed_objects.upper - i
			until
				i > highest_freezed_object.item
			loop
				eif_unfreeze (freezed_objects.item (i))
				i := i + 1
			end
			highest_freezed_object.set_item (freezed_objects.lower - 1)
		ensure
			all_unfreezed: highest_freezed_object.item < freezed_objects.lower
		end


feature {NONE} -- ISE routines for freezing/unfreezing objects

	eif_freeze (obj: ANY): POINTER is
			-- Make sure that `obj' will not be moved by GC.
		require
			garbage_collector_enabled: memory.collecting
			obj_not_void: obj /= Void
		external
			"C [macro <eif_eiffel.h>] (EIF_OBJECT): EIF_REFERENCE"
		alias
			"eif_freeze"
		end

	eif_unfreeze (ptr: POINTER) is
			-- Object at address `ptr' can be handled
			-- and moved by the GC now.
		require
			garbage_collector_enabled: memory.collecting
			ptr_not_null: ptr /= default_pointer
		external
			"C [macro <eif_eiffel.h>] (EIF_REFERENCE)"
		alias
			"eif_unfreeze"
		end

	memory: expanded MEMORY


feature {NONE} -- Freezed object remembering

	freezed_objects: ARRAY [POINTER] is
			-- All freezed objects are stored in this buffer. Call
			-- `clear_freezed_objects' to free them.
			-- This list of objects is shared among all external
			-- helpers. Calling `clear_freezed_objects' on any of them, will
			-- unfreeze all freezed objects.
		once
			create Result.make (1, 16)
		ensure
			freezed_objects_not_void: freezed_objects /= Void
			no_nil_pointers: True -- for_each o in freezed_objects (lower..highest_freezed_object) it_holds o /= default_pointer
		end

	highest_freezed_object: INTEGER_REF is
			-- Index in `freezed_objects', to be used by `store_pointer'.
		once
			create Result
		ensure
			highest_freezed_object.item >= freezed_objects.lower -1 and then
			highest_freezed_object.item <= freezed_objects.upper
		end

	store_freezed_object (p: POINTER) is
			-- Cache pointer in `freezed_objects'
		require
			p_not_null: p /= default_pointer
			garbage_collector_enabled: memory.collecting
		do
			highest_freezed_object.set_item (highest_freezed_object.item + 1)
			freezed_objects.force (p, highest_freezed_object.item)
		ensure then
			pointer_stored: freezed_objects.item (highest_freezed_object.item) = p
			object_stored: highest_freezed_object.item = old highest_freezed_object.item + 1
		end


end
