indexing

	description: "Standard C text file."

	author: "Berend de Boer"
	date: "$Date: 2003/07/10 $"
	revision: "$Revision: #3 $"

class

	STDC_TEXT_FILE

inherit

	STDC_FILE

	EPX_TEXT_INPUT_STREAM
		rename
			close as close_for_reading,
			end_of_input as eof,
			is_closable as is_closable_for_reading,
			name as filename
		undefine
			is_closable_for_reading,
			rewind
		end


creation

	create_read_write,
	create_write,
	open,
	open_append,
	open_read,
	open_read_write,
	attach_to_stream


feature -- Text specific routines

	chop is
		obsolete "Use chop_last_string instead."
		do
			chop_last_string
		end

	chop_last_string is
		obsolete "There is no longer a need for this function. read_string no longer reads line. Use read_line to read lines. read_line removes the end-of-line character(s) automatically."
			-- remove newline character (if any) from `last_string'
			-- only when the last characters from last_string are %N or
			-- %R%N, they're removed.
		do
			if last_string.count > 0 and then
				last_string.item (last_string.count) = '%N' then
				last_string.remove (last_string.count)
				if last_string.count > 0 and then
					last_string.item (last_string.count) = '%R' then
					last_string.remove (last_string.count)
				end
			end
		end

	put_nl, put_newline, write_nl, write_newline is
			-- write a single newline
		do
			write_string ("%N")
		end


feature -- Reading

	gets (bytes: INTEGER) is
		obsolete "Use read_line instead"
		do
			read_line
		end

	get_string (bytes: INTEGER) is
		obsolete "Use read_line instead"
		do
			read_line
		end

feature -- Input

	max_line_length: INTEGER is
			-- Maximum line length used in `read_line'.
		do
			Result := 1024
		ensure
			max_line_length_positive: Result >= 1
		end

	read_line is
			-- Read characters from input stream until a line separator
			-- or end of file is reached. Make the characters that have
			-- been read available in `last_string' and discard the line
			-- separator characters from the input stream.
			-- Reads a maximum of `max_line_length' characters per line.
			-- The line should not have a '%U' character in it, because
			-- that is treated as end-of-file.
		local
			p: POINTER
		do
			assert_input_ok
			assert_buf_has_room (max_line_length)
			p := posix_fgets (gets_buf.ptr, gets_buf.capacity, stream)
			if p = default_pointer then
				eof_read := posix_feof (stream)
				if eof_read then
					last_string.wipe_out
				else
					raise_posix_error
				end
			else
				sh.set_string_from_pointer (last_string, gets_buf.ptr)
				if eof then
					-- We don't want the eof to be True here, user has to call
					-- gets again to make Eof True, fits better with Eiffel loops.
					clear_error
				end
			end
			-- `last_read' includes the end-of-line characters
			last_read := last_string.count

			-- Remove end-of-line characters
			if last_string.count > 0 and then
				last_string.item (last_string.count) = '%N' then
				last_string.remove (last_string.count)
				if last_string.count > 0 and then
					last_string.item (last_string.count) = '%R' then
					last_string.remove (last_string.count)
				end
			end
		end

	read_new_line is
			-- Read a line separator from input file.
			-- Make the characters making up the recognized
			-- line separator available in `last_string',
			-- or make `last_string' empty and leave the
			-- input file unchanged if no line separator
			-- was found.
		do
			if last_string = Void then
				create last_string.make (256)
			else
				last_string.wipe_out
			end
			read_character
			if not eof then
				if last_character = '%N' then
					last_string.append_character ('%N')
				else
					-- Put character back to input file.
					unread_character (last_character)
				end
			end
			-- Is setting `eof_read' good enough?
			eof_read := False
		end


feature {NONE}

	set_mode (a_mode: STRING) is
			-- set text mode
		require else
			binary_mode_flag_not_set: not is_binary_mode_specification (a_mode)
		local
			c: CHARACTER
		do
			c := a_mode.item (a_mode.count)
			inspect c
			when  'b',  'B' then
				do_raise ("It is not allowed to specify the binary flag %
								 %for text files.")
			when 't', 'T' then
				-- already ok
				mode := a_mode
			else
				-- it's not sure if every POSIX system allows this
				-- mode := a_mode + "t"
				mode := a_mode
			end
		end


end
